@@ -1,7 +1,6 @@
-/* mpc_get_prec -- returns the common precision of real and imaginary part, or 0
-if they differ
+/* mpc_get_prec -- returns the common precision of real and imaginary part, or 0 if they differ
 
-Copyright (C) 2007 Andreas Enge
+Copyright (C) INRIA, 2007, 2009, 2010
 
 This file is part of the MPC Library.
 
@@ -20,12 +19,11 @@ along with the MPC Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 MA 02111-1307, USA. */
 
-#include "mpc.h"
 #include "mpc-impl.h"
 
-mp_prec_t
-mpc_get_prec (mpc_t x)
+mpfr_prec_t
+mpc_get_prec (mpc_srcptr x)
 {
-  mp_prec_t precre = MPFR_PREC (MPC_RE (x));
-  return (MPFR_PREC (MPC_IM (x)) == precre ? precre : 0);
+  mpfr_prec_t precre = MPC_PREC_RE (x);
+  return (MPC_PREC_IM (x) == precre ? precre : 0);
 }
