@@ -1,6 +1,6 @@
 /* mpc_init3 -- Initialize a complex variable with given precisions.
 
-Copyright (C) 2002 Andreas Enge, Paul Zimmermann
+Copyright (C) INRIA, 2002, 2009
 
 This file is part of the MPC Library.
 
@@ -19,12 +19,10 @@ along with the MPC Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 MA 02111-1307, USA. */
 
-#include "gmp.h"
-#include "mpfr.h"
-#include "mpc.h"
+#include "mpc-impl.h"
 
 void
-mpc_init3 (mpc_t x, mp_prec_t prec_re, mp_prec_t prec_im)
+mpc_init3 (mpc_t x, mpfr_prec_t prec_re, mpfr_prec_t prec_im)
 {
   mpfr_init2 (MPC_RE(x), prec_re);
   mpfr_init2 (MPC_IM(x), prec_im);
