@@ -1,6 +1,6 @@
 /* mpc_proj -- projection of a complex number onto the Riemann sphere.
 
-Copyright (C) 2008 Philippe Th\'eveny
+Copyright (C) INRIA, 2008, 2009
 
 This file is part of the MPC Library.
 
@@ -19,13 +19,12 @@ along with the MPC Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 MA 02111-1307, USA. */
 
-#include "mpc.h"
 #include "mpc-impl.h"
 
 int
 mpc_proj (mpc_ptr a, mpc_srcptr b, mpc_rnd_t rnd)
 {
-  if (MPC_IS_INF (b))
+  if (mpc_inf_p (b))
     {
       /* infinities projects to +Inf +i* copysign(0.0, cimag(z)) */
       int inex;
@@ -38,8 +37,8 @@ mpc_proj (mpc_ptr a, mpc_srcptr b, mpc_rnd_t rnd)
           inex = -inex;
         }
 
-      return MPC_INEX (0, inex); 
-    } 
+      return MPC_INEX (0, inex);
+    }
   else
     return mpc_set (a, b, rnd);
 }
