@@ -1,6 +1,6 @@
 /* mpc_ceil_log2 - returns ceil(log(d)/log(2))
 
-Copyright 2004 Patrick Pelissier, Paul Zimmermann
+Copyright (C) INRIA, 2004, 2009, 2010
 
 This file is part of the MPC Library.
 
@@ -19,17 +19,14 @@ along with the MPC Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 MA 02111-1307, USA. */
 
-#include "gmp.h"
-#include "mpfr.h"
-#include "mpc.h"
 #include "mpc-impl.h"
 
 /* returns ceil(log(d)/log(2)) if d > 0 */
 /* Don't use count_leading_zeros since it is in longlong.h */
-unsigned long
-mpc_ceil_log2 (unsigned long d)
+mpfr_prec_t
+mpc_ceil_log2 (mpfr_prec_t d)
 {
-  unsigned long exp;
+  mpfr_prec_t exp;
 
   for (exp = 0; d > 1; d = (d + 1) / 2)
     exp++;
