@@ -1,7 +1,7 @@
 /* mpc_urandom -- Generate a random complex number uniformly distributed in
    the interval [0,1[.
 
-Copyright (C) 2008 Philippe Th\'eveny
+Copyright (C) INRIA, 2008, 2009
 
 This file is part of the MPC Library.
 
@@ -20,7 +20,7 @@ along with the MPC Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 MA 02111-1307, USA. */
 
-#include "mpc.h"
+#include "mpc-impl.h"
 
 int
 mpc_urandom (mpc_ptr a, gmp_randstate_t state)
